!= ͥǡեץ
!
!= dcpam-primitive main program for generation of initial data file
!
! Authors::   Yoshiyuki O. Takahashi, Yasuhiro MORIKAWA
!             modified by Shin-ichi Takehiro for primitive model.
! Version::   $Id: init_data_primitive_PST2004.f90,v 1.1.1.1 2010-08-17 05:24:50 takepiro Exp $
! Tag Name::  $Name:  $
! Copyright:: Copyright (C) GFD Dennou Club, 2008. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]
!

program init_data_primitive
  !
  ! <b>Note that Japanese and English are described in parallel.</b>
  !
  ! ͥǡեޤ. 
  !
  ! Initial data file is created. 
  !

  ! ⥸塼 ; USE statements
  !

!!$  ! ͥǡ (ꥹȥǡ) 
!!$  ! Prepare initial data (restart data)
!!$  !
!!$  use initial_data, only: SetInitData

  ! ꥹȥǡ
  ! Restart data input/output
  !
  use restart_file_io, only: RestartFileOutPut

  ! ̷ѥ᥿
  ! Kind type parameter
  !
  use dc_types, only: DP, &      ! ټ¿. Double precision. 
    &                 STRING, &  ! ʸ.       Strings. 
    &                 TOKEN      ! .   Keywords. 

  ! ʸ ; Declaration statements
  !
  implicit none

  ! ͽѿ (ƥå $ t-\Delta t $ , $ t $ , $ t+\Delta t $ )
  ! Prediction variables  (Step $ t-\Delta t $ , $ t $ , $ t+\Delta t $ )
  !
  real(DP), allocatable:: xyz_U (:,:,:)
                              ! $ u $ .   ®. Eastward wind
  real(DP), allocatable:: xyz_V (:,:,:)
                              ! $ v $ .   ®. Northward wind
  real(DP), allocatable:: xyz_Temp (:,:,:)
                              ! $ T $ .   . Temperature
  real(DP), allocatable:: xy_Ps (:,:)
                              ! $ p_s $ . ɽ̵. Surface pressure

  ! ѿ
  ! Work variables
  !

  ! ¹ʸ ; Executable statement
  !

  ! ץν (֥롼)
  ! Initialization for the main program (Internal subroutine)
  !
  call MainInit

  ! ͥǡκ
  ! Generate initial data
  !
  call SetInitData( &
    & xyz_U, xyz_V, xyz_Temp, xy_Ps )  ! (out)

  ! ͥǡν
  ! Output initial data
  !
  call RestartFileOutput( &
    & xyz_U, xyz_V, xyz_Temp, xy_Ps )  ! (in)

  ! ץνλ (֥롼)
  ! Termination for the main program (Internal subroutine)
  !
  call MainTerminate

contains

  !-------------------------------------------------------------------

  subroutine MainInit
    !
    ! ץν³. 
    !
    ! Initialization procedure for the main program. 
    !

    ! MPI
    !
    use mpi_wrapper, only : MPIWrapperInit

    use dc_message, only: MessageNotify

    ! ޥɥ饤
    ! Command line option parser
    !
    use option_parser, only: OptParseInit

    ! NAMELIST եϤ˴ؤ桼ƥƥ
    ! Utilities for NAMELIST file input
    !
    use namelist_util, only: NmlutilInit

    ! 
    ! Time control
    !
    use timeset, only: TimesetInit
                              ! ƥå $ t $ λ. Time of step $ t $. 

    ! ϥեδܾ
    ! Management basic information for output files
    ! 
    use fileset, only: FilesetInit

    ! ʻ
    ! Grid points settings
    !
    use gridset, only: GridsetInit, &
      &                imax, & ! ٳʻ. 
                               ! Number of grid points in longitude
      &                jmax, & ! ٳʻ. 
                               ! Number of grid points in latitude
      &                kmax    ! ľؿ. 
                               ! Number of vertical level

    ! ʪ
    ! Physical constants settings
    !
    use constants, only: ConstantsInit

    ! ɸǡ
    ! Axes data settings
    !
    use axesset, only: AxessetInit

    ! ꥹȥǡ
    ! Restart data input/output
    !
    use restart_file_io, only: RestartFileOpen, RestartFileGet

    ! ʸ ; Declaration statements
    !
    implicit none
    character(*), parameter:: prog_name = 'init_data'
                              ! ץ̾. 
                              ! Main program name
    character(*), parameter:: version = &
      & '$Name:  $' // &
      & '$Id: init_data_primitive_PST2004.f90,v 1.1.1.1 2010-08-17 05:24:50 takepiro Exp $'
                              ! ץΥС
                              ! Main program version
    character(STRING):: brief
                              ! ¹ԥեδʷ
                              ! Brief account of executable file

    ! ¹ʸ ; Executable statement
    !

    ! Initialize MPI
    !
    call MPIWrapperInit


    brief = 'Initial data generation'
    call MessageNotify( 'M', prog_name, 'Run: %c', c1 = trim(brief) )
    call MessageNotify( 'M', prog_name, '-- version = %c', c1 = trim(version) )

    ! ޥɥ饤
    ! Command line option parser
    !
    call OptParseInit(prog_name, brief)

    ! NAMELIST ե̾
    ! Input NAMELIST file name
    !
    call NmlutilInit

    ! 
    ! Time control
    !
    call TimesetInit

    ! ϥեδܾ
    ! Management basic information for output files
    ! 
    call FilesetInit

    ! ʻ
    ! Grid points settings
    !
    call GridsetInit

    ! ʪ
    ! Physical constants settings
    !
    call ConstantsInit

    ! ɸǡ
    ! Axes data settings
    !
    call AxessetInit

    ! ͽѿγ
    ! Allocation of prediction variables
    !
    allocate( xyz_U    (0:imax-1, 1:jmax, 1:kmax) )
    allocate( xyz_V    (0:imax-1, 1:jmax, 1:kmax) )
    allocate( xyz_Temp (0:imax-1, 1:jmax, 1:kmax) )
    allocate( xy_Ps    (0:imax-1, 1:jmax) )

    ! ͥǡ
    ! Initial data output
    !
    call RestartFileOpen( flag_init_data = .true. ) ! (in) optional

  end subroutine MainInit

  !-------------------------------------------------------------------

  subroutine MainTerminate
    !
    ! ץνλ³. 
    !
    ! Termination procedure for the main program. 
    !

    ! MPI
    !
    use mpi_wrapper, only : MPIWrapperFinalize

    ! 
    ! Time control
    !
    use timeset, only: TimesetClose

    ! ꥹȥǡ
    ! Restart data input/output
    !
    use restart_file_io, only: RestartFileClose

    ! ʸ ; Declaration statements
    !
    implicit none

    ! ¹ʸ ; Executable statement
    !

    ! ꥹȥǡե륯
    ! Close restart data input
    !
    call RestartFileClose

    ! λ
    ! Termination of time control
    !
    call TimesetClose

    ! ͽѿγղ
    ! Deallocation of prediction variables
    !
    deallocate( xyz_U    )
    deallocate( xyz_V    )
    deallocate( xyz_Temp )
    deallocate( xy_Ps    )

    ! Finalize MPI
    !
    call MPIWrapperFinalize


  end subroutine MainTerminate

  !-------------------------------------------------------------------

  subroutine SetInitData( &
    & xyz_U, xyz_V, xyz_Temp, xy_Ps )  ! (out)
    !
    ! ǡ
    !
    ! Setting initial values
    !

    ! ʻ
    ! Grid points settings
    !
    use gridset, only: &
      &                imax, & ! ٳʻ. 
                               ! Number of grid points in longitude
      &                jmax, & ! ٳʻ. 
                               ! Number of grid points in latitude
      &                kmax    ! ľؿ. 
                               ! Number of vertical level

    ! ʪ
    ! Physical constants settings
    !
    use constants, only: PI, Omega, RPlanet, GasRDry

    ! ɸǡ
    ! Axes data settings
    !
    use axesset, only: x_Lon, y_Lat, z_Sigma, y_Lat_Weight

    use wa_module, only : a_AvrLat_ya

    real(DP), intent(OUT) :: xyz_U (0:,:,:)
                                    ! $ u $ .   ®. Eastward wind
    real(DP), intent(OUT) :: xyz_V (0:,:,:)
                                    ! $ v $ .   ®. Northward wind
    real(DP), intent(OUT) :: xyz_Temp (0:,:,:)
                                    ! $ T $ .   . Temperature
    real(DP), intent(OUT) :: xy_Ps (0:,:)     
                                    ! $ p_s $ . ɽ̵. Surface pressure

    integer, parameter :: nmax=100
    integer   :: i, j, k, n
    real(DP)  :: lat, dlat

    real(DP)  :: z_Z(1:kmax)
    real(DP)  :: z_F(1:kmax)
    real(DP)  :: z_dFdZ(1:kmax)
    real(DP)  :: z_Temp(1:kmax)

    real(DP)  :: yz_Temp(1:jmax,1:kmax)

    real(DP), parameter :: u0  = 50.0d0
    real(DP), parameter :: H   = 7.34d3
    real(DP), parameter :: z0  = 22.0d3
    real(DP), parameter :: z1  = 30.0d3
    real(DP), parameter :: dz0 = 5.0d3
    real(DP), parameter :: p0  = 1.0d5

    real(DP), parameter :: That    = 1.0d0
    real(DP), parameter :: lambda0 = 0.0d0
    real(DP), parameter :: phi0    = PI/4
    real(DP), parameter :: alpha   = 1.0d0/3.0d0
    real(DP), parameter :: beta    = 1.0d0/6.0d0
    
    xyz_V    = 0.0D0
    xy_Ps    = p0

    z_Z    = - H*log(z_Sigma)
    z_F    = 0.5D0 * ( 1 - tanh((z_Z-z0)/dz0)**3 ) * sin(PI*z_Z/z1)
    !
    !  zonal velocity profile
    ! 
    do k=1,kmax
       do j=1,jmax
          do i=0,imax-1
             if ( y_Lat(j) > 0.0d0 ) then
                xyz_U(i,j,k) = u0 * sin(PI*sin(y_Lat(j))**2)**3 * z_F(k)
             else
                xyz_U(i,j,k) = 0.0d0
             endif
          end do
       end do
    end do

    !
    !  temprature profile
    ! 
    z_dFdZ =-1.5d0 * tanh((z_Z-z0)/dz0)**2 / cosh((z_Z-z0)/dz0)**2 / dz0 &
         &   * sin(PI*z_Z/z1) &
         & + 0.5D0 * (1 - tanh((z_Z-z0)/dz0)**3) &
         &   * cos(PI*z_Z/z1) * (PI/z1)

    do k=1,kmax
       yz_Temp(1,k) = 0.0D0
       do j=1,jmax-1
          yz_Temp(j+1,k) = yz_Temp(j,k)
          dlat =(y_Lat(j+1)-y_Lat(j))/(nmax+1)
          do n=0,nmax
             lat = y_Lat(j) + dlat * (n+0.5)/(nmax+1)
             if ( lat > 0.0d0 ) then
                yz_Temp(j+1,k) = yz_Temp(j+1,k) &
                     -H/GasRDry                 &
                     * (   RPlanet * 2*Omega*sin(lat) &
                         + 2 * u0*sin(PI*sin(lat)**2)**3*z_F(k) * tan(lat) ) &
                     * u0*sin(PI*sin(lat)**2)**3*z_dFdz(k) * dlat
             else
                yz_Temp(j+1,k) = yz_Temp(j+1,k)
             endif
          enddo
       enddo
    enddo

    z_Temp = a_AvrLat_ya(yz_Temp)

    do k=1,kmax
       do j=1,jmax
          do i=0,imax-1
             xyz_Temp(i,j,k) = yz_Temp(j,k)-z_Temp(k) + TempUSStandard(z_Z(k))
          enddo
       end do
    end do

    !
    ! adding temperature disturbance
    !
    do k=1,kmax
       do j=1,jmax
          do i=0,imax-1
             if ( x_Lon(i) <= PI ) then
                xyz_Temp(i,j,k) = xyz_Temp(i,j,k) &
                     + That * cosh((x_Lon(i)-lambda0)/alpha)**(-2) &
                            * cosh((y_Lat(j)-phi0)/beta)**(-2) 
             else
                xyz_Temp(i,j,k) = xyz_Temp(i,j,k) &
                     + That * cosh((x_Lon(i)-lambda0-2*PI)/alpha)**(-2) &
                            * cosh((y_Lat(j)-phi0)/beta)**(-2) 
             endif
          enddo
       end do
    end do

  end subroutine SetInitData

  function TempUSStandard(z)
    !
    ! ꥫɸ絤(COESA 1976)٤η׻
    !
    ! Compute U.S. Standard Atmosphere temperature
    ! 
    use dc_message, only: MessageNotify

    real(DP), intent(IN) :: z               ! $ z $ .   Height
    real(DP)             :: TempUSStandard  
                         ! ɸ絤 U.S. Standard Atmosphere temperature

    if ( z >= 0  .AND. z < 11.0d3 ) then
       TempUSStandard = 288.15 - 6.5d-3*z 
    else if ( z >= 11.0d3 .AND. z < 20.0d3 ) then
       TempUSStandard = 288.15 - 6.5d-3*11.0d3
    else if ( z >= 20.0d3 .AND. z < 32.0d3 ) then
       TempUSStandard = 288.15 - 6.5d-3*11.0d3 + 1.0d-3*(z-20.0d3)
    else if ( z >= 32.0d3 .AND. z < 47.0d3 ) then
       TempUSStandard = 288.15 - 6.5d-3*11.0d3 + 1.0d-3*(32.0d3-20.0d3) &
            & + 2.8d-3 * (z-32.0d3)
    else if ( z >= 47.0d3 .AND. z < 51.0d3 ) then
       TempUSStandard = 288.15 - 6.5d-3*11.0d3 + 1.0d-3*(32.0d3-20.0d3) &
            & + 2.8d-3 * (47.0d3-32.0d3)
    else if ( z >= 51.0d3 .AND. z < 71.0d3 ) then
       TempUSStandard = 288.15 - 6.5d-3*11.0d3 + 1.0d-3*(32.0d3-20.0d3) &
            & + 2.8d-3 * (47.0d3-32.0d3) - 2.8d3 * (z-51.0d3)
    else if ( z >= 71.0d3 .AND. z < 80.0d3 ) then
       TempUSStandard = 288.15 - 6.5d-3*11.0d3 + 1.0d-3*(32.0d3-20.0d3) &
            & + 2.8d-3 * (47.0d3-32.0d3) - 2.8d3 * (71.0d3-51.0d3) &
            & - 2.0d-3 * (z-71.0d3)
    else if ( z >= 80.0d3 ) then
       TempUSStandard = 288.15 - 6.5d-3*11.0d3 + 1.0d-3*(32.0d3-20.0d3) &
            & + 2.8d-3 * (47.0d3-32.0d3) - 2.8d3 * (71.0d3-51.0d3) &
            & - 2.0d-3 * (80.0d3-71.0d3)
    else
       call MessageNotify('E','TempUSStandard','z must be larger than zero')
    endif
  end function TempUSStandard

end program init_data_primitive
